REGION_NAMES = {
    'os': 'Orkney & Shetland',
    'he': 'Highland & Eilean Siar',
    'gr': 'Grampian',
    'ta': 'Tayside',
    'st': 'Strathclyde',
    'dg': 'Dumfries, Galloway, Lothian',
    'ni': 'Northern Ireland',
    'yh': 'Yorkshire & the Humber',
    'ne': 'Northeast England',
    'em': 'East Midlands',
    'ee': 'East of England',
    'se': 'London & Southeast England',
    'nw': 'Northwest England',
    'wm': 'West Midlands',
    'sw': 'Southwest England',
    'wl': 'Wales',
    'uk': 'UK',
}